'use strict';

var vscodeLanguageserverProtocol = require('vscode-languageserver-protocol');
var css = require('@master/css');
var getCssDataDocumentation = require('./get-css-data-documentation.cjs');
var sortCompletionItems = require('./sort-completion-items.cjs');
var getUtilityInfo = require('./get-utility-info.cjs');
var cssDataProvider = require('./css-data-provider.cjs');

function getMainCompletionItems(css$1 = new css.MasterCSS()) {
    const completionItems = [];
    const addedKeys = new Set();
    for (const eachDefinedRule of css$1.definedRules){
        if (eachDefinedRule.definition.type === css.SyntaxRuleType.Utility) {
            const { data, detail, docs } = getUtilityInfo(eachDefinedRule, css$1);
            const utilityName = eachDefinedRule.id.slice(1);
            completionItems.push({
                label: utilityName,
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Value,
                documentation: getCssDataDocumentation.getCSSDataDocumentation(data, {
                    generatedCSS: css.generateCSS([
                        utilityName
                    ], css$1),
                    docs
                }),
                detail
            });
        } else {
            const nativeProperties = cssDataProvider.provideProperties();
            const nativeCSSPropertyData = nativeProperties.find(({ name })=>name === eachDefinedRule.id);
            const eachCompletionItem = {
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Property,
                documentation: getCssDataDocumentation.getCSSDataDocumentation(nativeCSSPropertyData, {
                    docs: '/reference/' + css.isCoreRule(eachDefinedRule.id) && eachDefinedRule.id
                }),
                detail: nativeCSSPropertyData?.syntax
            };
            eachDefinedRule.keys.forEach((key)=>{
                addedKeys.delete(key);
                completionItems.push({
                    ...eachCompletionItem,
                    label: key + ':',
                    sortText: key,
                    command: {
                        title: 'triggerSuggest',
                        command: 'editor.action.triggerSuggest'
                    }
                });
            });
            /**
             * @example @ animation and ~ transition
             */ if (eachDefinedRule.definition?.sign && eachDefinedRule.definition.includeAnimations) {
                css$1.animations.forEach((animation, animationName)=>{
                    completionItems.push({
                        ...eachCompletionItem,
                        label: eachDefinedRule.definition.sign + animationName + '|1s',
                        kind: vscodeLanguageserverProtocol.CompletionItemKind.Value
                    });
                });
            }
            if (eachDefinedRule.definition?.ambiguousKeys?.length) {
                for (const ambiguousKey of eachDefinedRule.definition.ambiguousKeys){
                    if (addedKeys.has(ambiguousKey)) {
                        continue;
                    }
                    addedKeys.add(ambiguousKey);
                }
            }
        }
    }
    addedKeys.forEach((ambiguousKey)=>{
        /**
         * Ambiguous keys are added to the completion list
         * @example text: t:
         */ completionItems.push({
            kind: vscodeLanguageserverProtocol.CompletionItemKind.Property,
            detail: 'ambiguous key',
            label: ambiguousKey + ':',
            sortText: ambiguousKey,
            command: {
                title: 'triggerSuggest',
                command: 'editor.action.triggerSuggest'
            }
        });
    });
    if (css$1.config.components) {
        for(const componentClass in css$1.config.components){
            const componentTokens = css$1.components.get(componentClass);
            if (!componentTokens) continue;
            completionItems.push({
                label: componentClass,
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Value,
                documentation: getCssDataDocumentation.getCSSDataDocumentation({}, {
                    generatedCSS: css.generateCSS([
                        componentClass
                    ], css$1),
                    docs: '/guide/components'
                }),
                detail: componentTokens.join(' ') + ' (style)'
            });
        }
    }
    return sortCompletionItems(completionItems);
}

module.exports = getMainCompletionItems;
