'use strict';

var vscodeLanguageserverProtocol = require('vscode-languageserver-protocol');
var cssDataProvider = require('./css-data-provider.cjs');
var css = require('@master/css');
var getCssDataDocumentation = require('./get-css-data-documentation.cjs');
var sortCompletionItems = require('./sort-completion-items.cjs');

const SCOPED_VARIABLE_PRIORITY = 'aaaa';
const AMBIGUOUS_PRIORITY = 'bbbb';
const NATIVE_PRIORITY = 'ccccc';
const GLOBAL_VARIABLE_PRIORITY = 'zzzz';
function getValueCompletionItems(css$1 = new css.MasterCSS(), ruleKey) {
    const nativeProperties = cssDataProvider.provideProperties();
    const completionItems = [];
    const nativeKey = css$1.definedRules.find(({ keys })=>keys.includes(ruleKey))?.id;
    const nativePropertyData = nativeProperties.find(({ name })=>name === nativeKey);
    const generateVariableCompletionItem = (variable, { scoped } = {
        scoped: false
    })=>{
        const eachNativePropertyData = nativeProperties.find((x)=>x.name === variable.group) || nativePropertyData;
        const appliedValue = scoped ? variable.key : variable.name;
        const documentation = getCssDataDocumentation.getCSSDataDocumentation(eachNativePropertyData, {
            generatedCSS: css.generateCSS([
                ruleKey + ':' + appliedValue
            ], css$1),
            docs: '/reference/' + (eachNativePropertyData?.name || 'variables')
        });
        const completionItem = {
            label: variable.name,
            kind: vscodeLanguageserverProtocol.CompletionItemKind.Value
        };
        const conflicted = completionItems.find(({ label })=>label === variable.name);
        if (conflicted) {
            completionItem.label = '$(' + completionItem.label + ')';
            completionItem.documentation = getCssDataDocumentation.getCSSDataDocumentation(eachNativePropertyData, {
                generatedCSS: css.generateCSS([
                    ruleKey + ':' + completionItem.label
                ], css$1),
                docs: '/reference/' + (eachNativePropertyData?.name || 'variables')
            });
        } else {
            completionItem.documentation = getCssDataDocumentation.getCSSDataDocumentation(eachNativePropertyData, {
                generatedCSS: css.generateCSS([
                    ruleKey + ':' + appliedValue
                ], css$1),
                docs: '/reference/' + (eachNativePropertyData?.name || 'variables')
            });
        }
        if (variable.type === 'color') {
            if (Object.keys(variable.modes || {}).length) {
                completionItem.kind = vscodeLanguageserverProtocol.CompletionItemKind.Color;
                completionItem.detail = variable.name;
            } else {
                // todo: packages/core should support getTextByVariable(variable)
                const configKey = 'variables.' + (variable.group ? variable.group + '.' + variable.key : variable.name);
                const valueToken = variable.space && variable.value ? `${variable.space}(${variable.value.split(' ').join(',')})` : variable.value;
                // detail is shown in the detail pane
                // todo: variable.token should be recorded as original config variable
                completionItem.detail = String(configKey);
                completionItem.documentation = {
                    kind: 'markdown',
                    value: valueToken + '\n\n' + documentation?.value
                };
                completionItem.kind = vscodeLanguageserverProtocol.CompletionItemKind.Color;
                completionItem.sortText = 'color-' + variable.name.replace(/(.+?)-(\d+)/, (match, prefix, num)=>prefix + num.padStart(10, '0'));
            }
        } else if (variable.type === 'number') {
            if (variable.name.startsWith('-') && eachNativePropertyData?.syntax?.includes('absolute')) return;
            completionItem.detail = String(variable.name);
            completionItem.sortText = (variable.group || '') + (variable.value >= 0 ? String(variable.value).padStart(10, '0') : '-' + String(Math.abs(variable.value)).padStart(10, '0'));
        } else {
            completionItem.detail = String(variable.value || variable.name);
        }
        return completionItem;
    };
    for (const eachDefinedRule of css$1.definedRules){
        /**
         * Scoped variables
         * @example box: + content -> box-sizing:content
         */ if (eachDefinedRule.definition.key === ruleKey || eachDefinedRule.definition.subkey === ruleKey || eachDefinedRule.definition.ambiguousKeys?.includes(ruleKey)) {
            for(const variableName in eachDefinedRule.variables){
                if (completionItems.find(({ label })=>label === variableName)) continue;
                const variable = eachDefinedRule.variables[variableName];
                const completionItem = generateVariableCompletionItem(variable, {
                    scoped: true
                });
                if (completionItem) {
                    completionItem.label = variableName;
                    completionItem.sortText = SCOPED_VARIABLE_PRIORITY + variableName;
                    completionItem.detail = '(scope) ' + completionItem.detail;
                    completionItems.push(completionItem);
                }
            }
        }
        /**
         * @example animation:fade
         */ if (eachDefinedRule.keys.includes(ruleKey) && eachDefinedRule.definition.includeAnimations) {
            css$1.animations.forEach((animation, animationName)=>{
                const isNative = eachDefinedRule.definition.type && [
                    css.SyntaxRuleType.Native,
                    css.SyntaxRuleType.NativeShorthand
                ].includes(eachDefinedRule.definition.type);
                completionItems.push({
                    label: animationName,
                    kind: vscodeLanguageserverProtocol.CompletionItemKind.Value,
                    documentation: getCssDataDocumentation.getCSSDataDocumentation(undefined, {
                        generatedCSS: css.generateCSS([
                            ruleKey + ':' + animationName
                        ], css$1),
                        docs: '/reference/' + css.isCoreRule(eachDefinedRule.id) && eachDefinedRule.id
                    }),
                    detail: isNative ? eachDefinedRule.id + ': ' + animationName : animationName
                });
            });
        }
        /**
         * Ambiguous values
         * @example text: -> center, left, right, justify
         * @example t: -> center, left, right, justify
         */ if (eachDefinedRule.definition.ambiguousKeys?.includes(ruleKey) && eachDefinedRule.definition.ambiguousValues?.length) {
            const nativePropertyData = nativeProperties.find((x)=>x.name === eachDefinedRule.id);
            for (const ambiguousValue of eachDefinedRule.definition.ambiguousValues){
                if (typeof ambiguousValue !== 'string') continue;
                const nativeValueData = nativePropertyData?.values?.find((x)=>x.name === ambiguousValue);
                const isNative = eachDefinedRule.definition.type && [
                    css.SyntaxRuleType.Native,
                    css.SyntaxRuleType.NativeShorthand
                ].includes(eachDefinedRule.definition.type);
                completionItems.push({
                    label: ambiguousValue,
                    kind: vscodeLanguageserverProtocol.CompletionItemKind.Value,
                    sortText: AMBIGUOUS_PRIORITY + ambiguousValue,
                    documentation: getCssDataDocumentation.getCSSDataDocumentation({
                        ...nativeValueData || {},
                        // use nativePropertyData.reference because nativeValueData does not have references
                        references: nativePropertyData?.references
                    }, {
                        generatedCSS: css.generateCSS([
                            ruleKey + ':' + ambiguousValue
                        ], css$1),
                        docs: '/reference/' + css.isCoreRule(eachDefinedRule.id) && eachDefinedRule.id
                    }),
                    detail: isNative ? eachDefinedRule.id + ': ' + ambiguousValue : ambiguousValue
                });
            }
        }
    }
    /**
     * Native values
     */ if (nativeKey) {
        nativePropertyData?.values?.forEach((value)=>{
            if (completionItems.find((x)=>x.label === value.name) || nativePropertyData.name === 'font' && typeof +value.name === 'number' || value.name.includes(' ')) return;
            completionItems.push({
                label: value.name,
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Value,
                sortText: NATIVE_PRIORITY + (value.name.startsWith('-') ? 'zz' + value.name.slice(1) : value.name),
                documentation: getCssDataDocumentation.getCSSDataDocumentation({
                    ...value,
                    // use nativePropertyData.reference because nativeValueData does not have references
                    references: nativePropertyData?.references
                }, {
                    generatedCSS: css.generateCSS([
                        ruleKey + ':' + value.name
                    ], css$1),
                    docs: '/reference/' + nativeKey
                }),
                detail: nativeKey + ': ' + value.name
            });
        });
    }
    // global variables
    css$1.variables.forEach((variable)=>{
        const completionItem = generateVariableCompletionItem(variable);
        if (completionItem) {
            completionItem.sortText = GLOBAL_VARIABLE_PRIORITY + (completionItem.sortText || completionItem.label);
            completionItem.detail = '(global) ' + completionItem.detail;
            completionItems.push(completionItem);
        }
    });
    return sortCompletionItems(completionItems);
}

module.exports = getValueCompletionItems;
